function [cracklength, edges_all]  = CrackEdge(vidObj,SampleCrack,SampleFail,BBox_i, BBox_f)
%%
% Code should be run using CrackEdge_runner.m for best user experience

% Leigh Slyker 07/28/23
% Please cite DOI HERE when making use of this program. 

%% User variables setting

FrameJump = 15; % Code will calculate every *FrameJump* frames

frames = SampleCrack:FrameJump:SampleFail; % Create matrix of frames for analysis

ycrop = 25; % crop in from top and bottom by *ycrop* pixels

%% variable initialization

Ybound	= cell(length(frames),1);
Xbound	= cell(length(frames),1);
sampleBW= cell(length(frames),1);
all_Edges	= cell(length(frames),1);
A_s		= cell(length(frames),1);
A		= cell(length(frames),1);
cracklength = cell(length(frames),1);
Yslice = cell(length(frames),1);
yCrack = cell(length(frames),1);
Xslice = cell(length(frames),1);
A2 = cell(length(frames),1);
LB = cell(length(frames),1);
UB = cell(length(frames),1);
y_points =  cell(length(frames),1);
x_points =  cell(length(frames),1);
xCrack = cell(length(frames),1);
Area = cell(length(frames),1);
t_LB = cell(length(frames),1);
b_UB = cell(length(frames),1);
b_UB_inflx = cell(length(frames),1);
b_perimeter = cell(length(frames),1);
edges_t_j2 = cell(length(frames),1);
edges_b_i2 = cell(length(frames),1);
edges_t_i2 = cell(length(frames),1);

edges_b_j2 = cell(length(frames),1);
edges_all = cell(length(frames),1);
gap_edges_j = cell(length(frames),1);
gap_edges_i = cell(length(frames),1);
b_edges = cell(length(frames),1);
b_length = cell(length(frames),1);
t_length = cell(length(frames),1);

b_perim = cell(100,1);
b_edge = cell(100,1);
t_edge = cell(100,1);
t_perim = cell(100,1);

t_perimeter = cell(length(frames),1);
t_LB_inflx = cell(length(frames),1);
t_edges = cell(length(frames),1);
Area2 = cell(length(frames),1);
 


	for f = 1:length(frames) 
% 	for f = 1
	

	% read in crack frame from vidObj at frame f
	frame = frames(f);
	
	disp(['Now analyzing frame ',num2str(f),' of ',...
		num2str(length(frames))]); 
	
	crackframe = read(vidObj,frame);

	% calculate y and x bounds at f
	
	%% Y and X positions of lower bound (LB) and upper bound (UB) scale
	% from initial (BBox_i) to final (BBox_f) size

	LB_Y = (BBox_f(2)-BBox_i(2))/length(frames)*f+BBox_i(2);
	UB_Y = ((BBox_f(2)+BBox_f(4))-(BBox_i(2)+BBox_i(4)))/...
		length(frames)*f+(BBox_i(2)+BBox_i(4));
	
	LB_X = (BBox_f(1)-BBox_i(1))/length(frames)*f+BBox_i(1);
	UB_X = ((BBox_f(1)+BBox_f(3))-(BBox_i(1)+BBox_i(3)))/...
		length(frames)*f+(BBox_i(1)+BBox_i(3));
	
	Ybound{f} = round(LB_Y):round(UB_Y);
	Xbound{f} = round(LB_X):round(UB_X);
	
	%%
	

	
	% binarize sample image of crackframe within Y and X bounds
	sampleBW{f} = imbinarize(crackframe(Ybound{f},Xbound{f}));

	%find the largest area of the sample image (which will correspond to
	%the sample 
	A_s{f} = max(cell2mat(struct2cell(regionprops(sampleBW{f},'Area'))));

	%filter image to exclude regions other than sample (such as noise)
	sampleBW{f} = bwpropfilt(sampleBW{f}, 'Area', [A_s{f}-1,A_s{f}+1]);

 	%find edges of that sample area
	all_Edges{f} = edge(sampleBW{f});

% debug: correct sampleBW
	% 	figure
	%	imshow(sampleBW{f})

	
	
%% Y slicing and area determination to find crack tip Y-position

	% Crop image section and divide into Y slices for analysis
	yi = ycrop:5:size(sampleBW{f},1)-ycrop;
	for i = 1:length(yi)-1
		y1 = yi(i);
		y2 = yi(i+1);
		%divide sample area into slices for analysis
		Yslice{f}{i}(:,:) = sampleBW{f}(y1:y2,:); 
	end

	
	% regionprops area determination by Y slice 
	for i = 1:length(yi)-1
		A{f}{i} = regionprops(Yslice{f}{i},'area');
			if isempty(cell2mat(struct2cell(A{f}{i}))) == 1
				break
			else
		Area{f}(i) = max(cell2mat(struct2cell(A{f}{i})));
		end
	end

	if isempty(cell2mat(struct2cell(A{f}{i}))) == 1
			break
	end


	Area{f} = max(Area{f})-Area{f};
	[~,yCrack{f}] =  max(Area{f});

	[pks,~,w,~] = findpeaks(Area{f});

	if length(find(Area{f} == max(pks))) > 1

		spancheck = diff(find(Area{f} == max(pks)));

		if length(find(Area{f} == max(pks))) == length(spancheck == 1)+1
			yCrack{f} = mean([ yCrack{f} max(find(Area{f} == max(pks)))]);
		else
			disp('broken span')
		end

	end

	%%

	yCrack{f} = yCrack{f}/length(Area{f})*(length(sampleBW{f})-2*ycrop) + 2*ycrop;

	%% determine crack extent using 2*FWHM of area peak


	[~,max_i] = max(pks);
	FWHM = w(max_i);

	%expand crack extent width by factor of sample strain
	FWHM = FWHM*range(Ybound{f})/range(Ybound{1});
	
	LB{f} = round(yCrack{f}-3*FWHM);
	LB{f} = max([LB{f},1]);

	UB{f} = round(yCrack{f}+3*FWHM);
	UB{f} = min([UB{f},length(sampleBW{f})]);
	
	xi = 1:5:size(sampleBW{f},2);

	for i = 1:length(xi)-1
		x1 = xi(i);
		x2 = xi(i+1);

		Xslice{f}{i}(:,:) = imcomplement(sampleBW{f}(LB{f}:UB{f},x1:x2));
	end

	for i = 1:length(xi)-1

		A2{f}{i} = regionprops(Xslice{f}{i},'area');

		if isempty(A2{f}{i}) == 1
			break
		end

		Area2{f}(i) = max(cell2mat(struct2cell(A2{f}{i})));
	end

	[~,xCrack{f}] =  min(Area2{f});

	xCrack{f} = xCrack{f}/(length(xi))*size(sampleBW{f},2);

	%%

	Area2{f} = max(Area2{f})-Area2{f};

	xfirst = find((Area2{f}-min(Area2{f}))>0,1,'first');
	xlast = size(sampleBW{f},2);


		LB{f} = round(yCrack{f}-2*FWHM);
		LB{f} = max([LB{f},1]);
		UB{f} = round(yCrack{f}+2*FWHM);
		UB{f} = min([UB{f},length(sampleBW{f})]);

	clear rhs_check
	rhs_check = regionprops(imcomplement(sampleBW{f}(LB{f}:UB{f},round(xCrack{f})-5:xlast)));

	while length(rhs_check) > 1 
		xlast = xlast-1;
		clear rhs_check
		rhs_check = regionprops(imcomplement(sampleBW{f}(LB{f}:UB{f},round(xCrack{f})-5:xlast)));
	end

		if isempty(xlast) == 1 || isempty(xfirst) ==1 
			disp(['Failed to find xCrack at: f = ',num2str(f)])
			imshow(sampleBW{f}(round(yCrack{f}-2*FWHM):round(yCrack{f}+2*FWHM),:))
			return

		end

	%% top edge dimension surface generation
	y_points{f} = linspace(1,4);
	x_points{f} = 1:(floor(xCrack{f}-1)); 

	for i = 1:100
		t_i = y_points{f}(i);

		for j = 1:length(x_points{f})-1

			x_i = x_points{f}(j);

			t_LB{f}(i) = round(yCrack{f}-t_i*FWHM);
			t_LB{f}(i) = max([t_LB{f}(i),1]);
			t_UB = floor(yCrack{f});

			t_edge{i}{j} = imcomplement(sampleBW{f}(t_LB{f}(i):t_UB,x_i:xlast-5));
			t_perim{i}{j} = regionprops(t_edge{i}{j},'perimeter');
			if isempty(t_perim{i}{j}) == 0
				t_length{f}(i,j) = max(cell2mat(struct2cell(t_perim{i}{j}))) - size(t_edge{i}{j},1)- size(t_edge{i}{j},2);
			end
		end
	end

	%% top edge dimension surface analysis

	[~,ytop_i] = max(t_length{f},[],'all', 'linear'); 

	[r,~] = ind2sub([100,length(x_points)],ytop_i);


	dif_t = t_length{f}(r,:) - linspace(max(t_length{f}(r,:)),...
		min(t_length{f}(r,:)),length(t_length{f}(r,:)));

	[~,inflx] = max(dif_t);

	x_t_inflx =  min(inflx+1,size(t_length{f},2));
	[~, y_t_inflx] = max(t_length{f}(:,x_t_inflx));

	y_t_i = y_points{f}(y_t_inflx);
	t_LB_inflx{f} = t_LB{f}(y_t_inflx); 

	t_perimeter{f} = max(cell2mat(struct2cell(t_perim{y_t_inflx}{x_t_inflx})));
	%% top edge dimension check
	% 
	% 	y_i = y_points(38);
		x_t_i = x_points{f}(x_t_inflx);
	% figure
	% imshow(sampleBW{f}(round(yCrack{f}-y_t_i*FWHM):round(yCrack{f}),x_i:xlast));

	%% top edge surface plot
	% 
	% surf(x_points,y_points,t_length{f})
	% figure
	% surf(t_length{f})

	% plot(ypoints,t_length{f})

	% ytop = yCrack-y_points(ytop_i)*FWHM;

	%%

% 	t_i = y_t_i;

	x_i = x_t_i;

	% d = sample{f}(round(yCrack{f}-t_i*FWHM):round(yCrack{f}),x_i:xlast);

	% t_edges{f} = edge(imcomplement(sampleBW{f}(ceil(yCrack{f}-t_i*FWHM):round(yCrack{f}),x_i:xlast)));
	t_edges{f} = all_Edges{f}(t_LB_inflx{f}:round(yCrack{f}),x_i:xlast);

	[edges_t_i,edges_t_j] = find(t_edges{f} == 1);

	% figure
	% imshow(d), hold on
	% plot(edges_t_j,edges_t_i,'r.')

	% 
	% b_edge = imcomplement(sampleBW(round(yCrack):round(yCrack+20),xfirst:xlast));
	% b_perim = regionprops(b_edge,'perimeter');
	% b_length{f} = b_perim.Perimeter - size(b_edge,1)- size(b_edge,2);
	% 
	% cracklength = t_length{f} + b_length{f};

	%% bottom edge dimension surface generation

	y_points{f} = linspace(1,4);
	x_points{f} = 1:(floor(xCrack{f}-1)); 

	for i = 1:100
		b_i = y_points{f}(i);

		for j = 1:length(x_points{f})-1

			x_i = x_points{f}(j);

			b_LB = round(yCrack{f});
			b_UB{f}(i) = round(yCrack{f}+b_i*FWHM);
			b_UB{f}(i) = min([b_UB{f}(i),length(sampleBW{f})]);

			b_edge{i}{j} = imcomplement(sampleBW{f}(b_LB:b_UB{f}(i),x_i:xlast-5));
			b_perim{i}{j} = regionprops(b_edge{i}{j},'perimeter');
			if isempty(b_perim{i}{j}) == 0
				b_length{f}(i,j) = max(cell2mat(struct2cell(b_perim{i}{j})))- size(b_edge{i}{j},1)- size(b_edge{i}{j},2);
			end
		end
	end

	%% bottom edge dimension surface analysis

	[~,ybot_i] = max(b_length{f},[],'all', 'linear'); 

	[r,~] = ind2sub([100,length(x_points)],ybot_i);


	dif_b = b_length{f}(r,:) - linspace(max(b_length{f}(r,:)),...
		min(b_length{f}(r,:)),length(b_length{f}(r,:)));

	[~,inflx] = max(dif_b);
	% 
	x_b_inflx = min(inflx+1,size(b_length{f},2));
	[~, y_b_inflx] = max(b_length{f}(:,x_b_inflx));
	% 
% 	y_b_i = y_points{f}(y_b_inflx);
	b_UB_inflx{f} = b_UB{f}(y_b_inflx);

	b_perimeter{f} = max(cell2mat(struct2cell(b_perim{y_b_inflx}{x_b_inflx})));

	%% bottom edge dimension check
	% 
	% 	y_b_i = y_points(75);
		x_b_i = x_points{f}(x_b_inflx);
	% figure
	% imshow(sampleBW{f}(round(yCrack{f}):round(yCrack{f}+y_b_i*FWHM),x_i:xlast),...
	% 	'InitialMagnification',500);

	%% bottom edge surface plot

	% % surf(x_points,y_points,t_length{f})
	% figure
	% surf(b_length{f})

	%%


% 	b_i = y_b_i;
	x_i = x_b_i;

% 	d = sample{f}(round(yCrack{f}):b_UB_inflx{f},x_i:xlast);
	% b_edges{f} = edge(imcomplement(sampleBW{f}(floor(yCrack{f}:round(yCrack{f}+b_i*FWHM)),x_i:xlast)));
	b_edges{f} = all_Edges{f}(floor(yCrack{f}):b_UB_inflx{f},x_i:xlast);

	[edges_b_i, edges_b_j] = find(b_edges{f} == 1);

	% figure
	% imshow(d), hold on

	% plot(edges_b_j,edges_b_i,'r.')
	% imshow(sampleBW(round(yCrack:round(yCrack+b_i*FWHM)),x_i:xlast))


	% figure
	% imshow(sampleBW)
	% ginput(1)
	% close

	%%

	edges_t_j2{f} = edges_t_j + x_t_i;
	edges_t_i2{f} = edges_t_i - y_t_i*FWHM + yCrack{f};

	edges_b_j2{f} = edges_b_j + x_b_i;
	edges_b_i2{f} = edges_b_i + yCrack{f};
	
	
	if		isempty(edges_t_j2{f}) == 1 ||...
			isempty(edges_t_i2{f}) == 1 
				disp(['Top edge detection failed for frame ',...
					num2str(f),' of ', num2str(length(frames))]);
				cracklength{f} = [];
	elseif  isempty(edges_b_j2{f}) == 1 ||...
			isempty(edges_b_i2{f}) == 1
				disp(['Bottom edge detection failed for frame ',...
					num2str(f),' of ', num2str(length(frames))]);
				cracklength{f} = [];
	else
		cracklength{f} = t_perimeter{f} + b_perimeter{f} + ... %sum of top and bottom perimeters
			((max(edges_b_j2{f}) - max(edges_t_j2{f}))^2 + ... %distance between top and bottom edge extents
			(min(edges_b_i2{f}) - max(edges_t_i2{f}))^2)^(1/2);

	
	



	%%
	
	% 	crackcheck = insertObjectAnnotation(sample{f},'rectangle',...
	% 		[x_t_i round(yCrack{f}-y_t_i*FWHM) (xlast-min([x_t_inflx x_b_inflx])) round(y_b_i*FWHM+y_t_i*FWHM)],...
	% 		['Crack Length: ',num2str(cracklength{f}),' px'],'FontSize',10);
	%	crackcheck = insertObjectAnnotation(crackcheck,'rectangle',...
	%		[x_t_i round(yCrack-y_t_i*FWHM) (xlast-min([x_t_inflx x_b_inflx])) round(y_b_i*FWHM+y_t_i*FWHM)],'Crack');
	% 
	% 	figure
	% 	imshow(crackcheck); hold on


	gap_edges_i{f} =  max(edges_t_i2{f}):min(edges_b_i2{f});
	gap_edges_j{f} = linspace(max(edges_b_j2{f}),max(edges_t_j2{f}),length(gap_edges_i{f}));

	
% 	if length(gap_edges_i{f}) >= 1
	edges_all{f} = {edges_t_j2{f},gap_edges_j{f},edges_b_j2{f}...
			edges_t_i2{f},gap_edges_i{f},edges_b_i2{f}};
		
% 	else
% 		edges_all{f}{1} = [edges_t_j2{f}',edges_b_j2{f}'];
% 		edges_all{f}{2} = [edges_t_i2{f}',edges_b_i2{f}'];
% 	end
	end
	
		
	end
	
	%% save data to filename 
	
	savepath = strrep(vidObj.Path,'\videos','');
	cd(savepath)
	
	
	filename = strrep(vidObj.Name,'.MP4','_cracklength_bioeng');

	save(filename)
	disp(['Saved: ',filename])


	cd('\\LAPTOP-GHUKL5K6\Users\leigh\Documents\Desktop\Tough Hydrogels\Crack Edge Analysis')
	
	
end