function [filename] = Stress_Strain_Finder(vidObj,testdate,SampleCrack,SampleFailure)
%Stress Strain Matching Code to match video frames with stress-strain data

% Leigh Slyker 07/28/23
% Please cite DOI HERE when making use of this program. 



homedir = cd;

%% Video/Data import


cd(testdate)

datafiles = ls('*.mat');


datafile = datafiles(1,:);
	
find = strfind(datafile,'crackprop');
if isempty(find) == 0
	postproc = 1;
	disp('Post-processing mode autoselected!')
else
	postproc = 0;
end



%% Tensile Data import


cd(testdate)

load(datafile)

disp(vidObj.name)

[indx,~] = listdlg('PromptString',{'Select a test from:',...
					strrep(datafile,'.mat','')},...
					'SelectionMode','single','ListString',sheets);




%%
Area = A(indx);
RawStressData = S2{indx};
RawDisplacementData = e{indx};


%% Sample Props		

Thickness = textread('thickness.txt','%f','delimiter','\n','whitespace','');
Thickness = Thickness(indx);

Height = L0(indx); %mm
Height = Height/1000; % m

StrainRate = 0.167; % mm/s



%% Keyframe Identification

if postproc == 0 
%%
% f = msgbox('Find test start frame. Close window when done');
% waitfor(f)
% 
% m = implay(vidfile,60);
% waitfor(m)
% prompt = {'Start Frame:'};
% dlgtitle = 'Input';
% dims = [1 35];
% definput = {'100'};
% TestStart = str2double(inputdlg(prompt,dlgtitle,dims,definput));

TestStart = 200;
%%
% f = msgbox('Find fracture initiation frame');
% waitfor(f)
% 
% m = implay(vidfile,30);
% waitfor(m)
% prompt = {'Fracture Initiation Frame:'};
% dlgtitle = 'Input';
% dims = [1 35];
% definput = {'1000'};
% SampleCrack = str2double(inputdlg(prompt,dlgtitle,dims,definput));

%%
% f = msgbox('Find failure frame');
% waitfor(f)
% 
% m = implay(vidfile,30);
% waitfor(m)
% prompt = {'Failure Frame:'};
% dlgtitle = 'Input';
% dims = [1 35];
% definput = {'2000'};
% SampleFailure = str2double(inputdlg(prompt,dlgtitle,dims,definput));

end

%%
%Enter some Parameters
% TestStart = Start;
% SampleFailureFrame = 2000;

%choose start and end frame frame interval
% Start = Start;

FrameJump = 15;
% N =(SampleFailure-Start)/FrameJump;

% TimetoFailureFrame = SampleFailureFrame/vidObj.FrameRate;


%% Sample Dimension Normalization - input

if postproc == 0 
	
%establish normalized width

vidFrameNorm = read(vidObj,TestStart);

g = insertText(vidFrameNorm,[1 1],'Sample Width Normalization','FontSize',50);
imshow(g)
[pointX,pointY] = ginput(2);
close

%establish normalized height

vidFrameNorm = read(vidObj,TestStart);

g = insertText(vidFrameNorm,[1 1],'Sample Height Normalization','FontSize',50);
imshow(g)
[point2X,point2Y] = ginput(2);
close


vidFrameNorm = read(vidObj,SampleCrack);

g = insertText(vidFrameNorm,[1 1],'Crack Propagation Strain','FontSize',50);
imshow(g)
[point3X,point3Y] = ginput(2);
close

pointX2=NaN(1,2);
pointY2=NaN(1,2);
vidFrameNorm = read(vidObj,SampleFailure);

g = insertText(vidFrameNorm,[1 1],'Failure Strain','FontSize',50);
imshow(g)
[point4X,point4Y] = ginput(2);
close

end

%% Sample Dimension Normalization - analysis & output
normalization_h=((point2X(1)-point2X(2))^2+(point2Y(1)-point2Y(2))^2)^0.5; % pixels
normalization_w=((pointX(1)-pointX(2))^2+(pointY(1)-pointY(2))^2)^0.5; % pixels

scale = Height/normalization_h; % m/pixels
Width = normalization_w*scale; % m

normalization_p=((point3X(1)-point3X(2))^2+(point3Y(1)-point3Y(2))^2)^0.5; % pixels
CritStrain = (normalization_p-normalization_h)/normalization_h; % percent

normalization_f=((point4X(1)-point4X(2))^2+(point4Y(1)-point4Y(2))^2)^0.5; % pixels
FinalStrain = (normalization_f-normalization_h)/normalization_h; % percent

disp(['Sample width = ',num2str(Width*1000),' mm'])
disp(' ')
disp(['Critical Strain = ',num2str(CritStrain)])
disp(['Failure strain = ',num2str(FinalStrain)])

%% 
StressF = RawStressData(RawDisplacementData <= FinalStrain ...
					& RawDisplacementData >= CritStrain); %Pa
StrainF = RawDisplacementData(RawDisplacementData <= FinalStrain...
					& RawDisplacementData >= CritStrain); % percent


%% Data analysis

DispLength = length(SampleCrack:FrameJump:SampleFailure);

Disp = linspace(CritStrain,FinalStrain,DispLength);

for i = 1:DispLength
	Stress(i) = min(StressF(abs((StrainF-Disp(i)))./Disp(i) <= 0.02));
	Strain(i) = min(StrainF(abs((StrainF-Disp(i)))./Disp(i) <= 0.02));
end

Force = Stress*Area; %N
Force_s = smoothdata(Force,'gaussian',3);

Ext = Strain*Height;

Stiffness = Stress./Strain; %Pa
CritExt = CritStrain*Height;

%%

dF = diff(Force_s);
d2F = diff(smoothdata(dF,'gaussian'));
dExt = diff(Ext);
d2Ext = diff(dExt);


E = Force./Ext;

E_s = smoothdata(E,'gaussian');

dE = diff(E_s);


dE_s = smoothdata(dE,'gaussian',2);


CritExt_new = min(Ext(dE_s <= 0));


%% Save analysis datafile


cd(testdate)

filename = strrep(datafile,'.mat',['_StrainMatch_',strrep(vidObj.name,'.MP4',''),'.mat']);

save(filename)

cd(homedir)

end

